parameters num_remotes;

SET OPTION public.Global_Database_ID = 0;
SET permanent;

/* 
 * Everything is owned by cons.  Later on we will make cons into a group
 * cons is also the publisher
 */

GRANT CONNECT TO cons IDENTIFIED BY sql;
GRANT DBA, RESOURCE, REMOTE DBA TO cons;
GRANT PUBLISH TO cons;
CONNECT USER cons IDENTIFIED BY sql;
CREATE REMOTE MESSAGE TYPE FILE ADDRESS 'cons';

/* 
 * Create Tables here 
 */

CREATE TABLE admin (
        a_pkey                        bigint NOT NULL DEFAULT GLOBAL AUTOINCREMENT(1000000),
        textcol                       char(64) NULL,
        PRIMARY KEY (a_pkey)
);

CREATE TABLE parent (
        p_pkey                        bigint NOT NULL DEFAULT GLOBAL AUTOINCREMENT(1000000),
        rep_id                        integer NOT NULL,
        textcol                       char(64) NULL,
        PRIMARY KEY (p_pkey)
);

CREATE TABLE child (
        c_pkey                        bigint NOT NULL DEFAULT GLOBAL AUTOINCREMENT(1000000),
        p_pkey                        bigint NOT NULL,
        textcol                       char(64) NULL,
        PRIMARY KEY (c_pkey)
);

ALTER TABLE child ADD NOT NULL FOREIGN KEY (p_pkey) REFERENCES parent (p_pkey);

/* 
 * Create publications here
 */

CREATE PUBLICATION p1 (
        TABLE cons.admin,
        TABLE cons.parent subscribe by rep_id,
        TABLE cons.child subscribe by (select rep_id from cons.parent where cons.parent.p_pkey = cons.child.p_pkey)
);

CREATE TRIGGER bu_repid_parent BEFORE UPDATE OF rep_id ON cons.parent 
REFERENCING OLD AS old_row NEW AS new_row FOR EACH ROW
BEGIN
  UPDATE cons.child PUBLICATION p1
    OLD SUBSCRIBE BY old_row.rep_id
    NEW SUBSCRIBE BY new_row.rep_id
    WHERE cons.child.p_pkey = new_row.p_pkey;
END;


/* 
 * Now make cons a group and give the group access
 * to all of the objects owned by cons
 */

GRANT GROUP TO cons;
GRANT SELECT,INSERT,UPDATE,DELETE ON parent TO cons;
GRANT SELECT,INSERT,UPDATE,DELETE ON child TO cons;
GRANT SELECT,INSERT,UPDATE,DELETE ON admin TO cons;

/* 
 * Create Remote Users  
 */
 
CREATE PROCEDURE MakeUsers ( in num integer )
BEGIN
  DECLARE i INTEGER;
  
  SET i = 1;
  WHILE ( i <= num ) LOOP
    EXECUTE IMMEDIATE 'GRANT CONNECT TO rem' || i || ' IDENTIFIED BY sql';
    EXECUTE IMMEDIATE 'GRANT REMOTE TO rem' || i || ' TYPE FILE ADDRESS ''rem' || i || '''';
    EXECUTE IMMEDIATE 'GRANT REMOTE DBA TO rem' || i ;
    EXECUTE IMMEDIATE 'GRANT MEMBERSHIP IN GROUP cons TO rem' || i ;
    EXECUTE IMMEDIATE 'CREATE SUBSCRIPTION TO p1(''' || i || ''') FOR rem' || i || '';
    EXECUTE IMMEDIATE 'START SUBSCRIPTION TO p1(''' || i || ''') FOR rem' || i || '';
    SET i = i + 1;
  END LOOP;
END;

CALL MakeUsers ( {num_remotes} );

create procedure rep_data (in iter integer default 1, in do_commit integer default 0) 
begin
  declare loop_var integer;
  declare par_value bigint;
  
  set loop_var = 0;
  while loop_var < iter loop
    insert into cons.admin values (DEFAULT, uuidtostr(newid()) );
    insert into cons.parent values (DEFAULT, cast(rand()*{num_remotes} as integer) + 1, uuidtostr(newid()) );
    set par_value = @@identity;
    insert into cons.child values (DEFAULT, par_value, uuidtostr(newid()) );
    insert into cons.child values (DEFAULT, par_value, uuidtostr(newid()) );
    insert into cons.child values (DEFAULT, par_value, uuidtostr(newid()) );
    set loop_var=loop_var+1;
    if do_commit = 1 then
      commit;
    end if;
  end loop;
end;

call rep_data ( {num_remotes}0 );
commit;
